@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2009-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -33,12 +33,13 @@ from mailman.interfaces.bans import IBanManager
 from mailman.interfaces.listmanager import ListDeletingEvent
 from mailman.interfaces.mailinglist import SubscriptionPolicy
 from mailman.interfaces.member import (
-    AlreadySubscribedError, MemberRole, MembershipIsBannedError,
-    NotAMemberError)
+    AlreadySubscribedError, DeliveryMode, DeliveryStatus, MemberRole,
+    MembershipIsBannedError, NotAMemberError)
 from mailman.interfaces.pending import IPendable, IPendings
 from mailman.interfaces.subscriptions import (
     ISubscriptionManager, ISubscriptionService,
-    SubscriptionConfirmationNeededEvent, SubscriptionPendingError, TokenOwner,
+    SubscriptionConfirmationNeededEvent, SubscriptionInvitationNeededEvent,
+    SubscriptionPendingError, TokenOwner,
     UnsubscriptionConfirmationNeededEvent)
 from mailman.interfaces.template import ITemplateLoader
 from mailman.interfaces.user import IUser
@@ -167,21 +168,42 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         'pre_approved',
         'pre_confirmed',
         'pre_verified',
+        'invitation',
         'address_key',
         'subscriber_key',
         'user_key',
         'token_owner_key',
         'send_welcome_message',
+        'delivery_mode',
+        'delivery_status',
         )
 
     def __init__(self, mlist, subscriber=None, *,
                  pre_verified=False, pre_confirmed=False, pre_approved=False,
-                 send_welcome_message=None):
+                 invitation=False, send_welcome_message=None,
+                 delivery_mode=None, delivery_status=None):
         super().__init__(mlist, subscriber)
-        self.pre_verified = pre_verified
-        self.pre_confirmed = pre_confirmed
-        self.pre_approved = pre_approved
+        # An invitation is already supposed to be "pre_approved" and by
+        # confirming the invite a user will "confirm" and "verify" their
+        # subscription, hence these values are set to `True` if this is
+        # an invitation.
+        self.pre_verified = invitation or pre_verified
+        self.pre_confirmed = invitation or pre_confirmed
+        self.pre_approved = invitation or pre_approved
+        self.invitation = invitation
         self.send_welcome_message = send_welcome_message
+        # For enum types, we use the string values here instead of the enum
+        # objects because they are easier to serialize when the workflow is
+        # saved in the database. When setting the values, we restore back to
+        # Enum types.
+        if delivery_status is not None:
+            self.delivery_status = delivery_status.name
+        else:
+            self.delivery_status = None
+        if delivery_mode is not None:
+            self.delivery_mode = delivery_mode.name
+        else:
+            self.delivery_mode = None
 
     def _step_sanity_checks(self):
         # Ensure that we have both an address and a user, even if the address
@@ -235,6 +257,10 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         self.push('verification_checks')
 
     def _step_verification_checks(self):
+        # If this is an invitation, send it now.
+        if self.invitation:
+            self.push('send_invitation')
+            return
         # Is the address already verified, or is the pre-verified flag set?
         if self.address.verified_on is None:
             if self.pre_verified:
@@ -331,6 +357,13 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         # We can immediately subscribe the user to the mailing list.
         self.member = self.mlist.subscribe(
             self.subscriber, send_welcome_message=self.send_welcome_message)
+        # Set member attributes.
+        if self.delivery_mode:
+            self.member.preferences.delivery_mode = DeliveryMode[
+                self.delivery_mode]
+        if self.delivery_status:
+            self.member.preferences.delivery_status = DeliveryStatus[
+                self.delivery_status]
         assert self.token is None and self.token_owner is TokenOwner.no_one, (
             'Unexpected active token at end of subscription workflow')
 
@@ -344,6 +377,16 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         # Now we wait for the confirmation.
         raise StopIteration
 
+    def _step_send_invitation(self):
+        self._set_token(TokenOwner.subscriber)
+        self.push('do_confirm_verify')
+        self.save()
+        # Triggering this event causes the confirmation message to be sent.
+        notify(SubscriptionInvitationNeededEvent(
+            self.mlist, self.token, self.address.email))
+        # Now we wait for the confirmation.
+        raise StopIteration
+
     def _step_do_confirm_verify(self):
         # Restore a little extra state that can't be stored in the database
         # (because the order of setattr() on restore is indeterminate), then
@@ -366,7 +409,7 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
                      if self.mlist.subscription_policy in (
                          SubscriptionPolicy.moderate,
                          SubscriptionPolicy.confirm_then_moderate,
-                         )
+                         ) and not self.invitation
                      else 'do_subscription')
         self.push(next_step)
 
@@ -396,7 +439,9 @@ class UnSubscriptionWorkflow(_SubscriptionWorkflowCommon):
         self.pre_approved = pre_approved
 
     def _step_subscription_checks(self):
-        assert self.mlist.is_subscribed(self.subscriber)
+        # XXX assert self.member is not None is sufficient, but ...
+        assert (self.member is not None and
+                self.mlist.is_subscribed(self.member.subscriber))
         self.push('confirmation_checks')
 
     def _step_confirmation_checks(self):
@@ -518,14 +563,19 @@ class SubscriptionManager:
 
     def register(self, subscriber=None, *,
                  pre_verified=False, pre_confirmed=False, pre_approved=False,
-                 send_welcome_message=None):
+                 invitation=False, send_welcome_message=None,
+                 delivery_mode=None, delivery_status=None):
         """See `ISubscriptionManager`."""
         workflow = SubscriptionWorkflow(
             self._mlist, subscriber,
             pre_verified=pre_verified,
             pre_confirmed=pre_confirmed,
             pre_approved=pre_approved,
-            send_welcome_message=send_welcome_message)
+            invitation=invitation,
+            send_welcome_message=send_welcome_message,
+            delivery_mode=delivery_mode,
+            delivery_status=delivery_status,
+            )
         list(workflow)
         return workflow.token, workflow.token_owner, workflow.member
 
@@ -566,7 +616,19 @@ class SubscriptionManager:
 
 
 def _handle_confirmation_needed_events(event, template_name):
-    subject = 'confirm {}'.format(event.token)
+    # This function handles sending the confirmation email to the user
+    # for both subscriptions requiring confirmation and invitations
+    # requiring acceptance.
+    if template_name.endswith(':invite'):
+        subject = _('You have been invited to join the '
+                    '$event.mlist.fqdn_listname mailing list.')
+    elif template_name.endswith(':unsubscribe'):
+        subject = _('Your confirmation is needed to leave the '
+                    '$event.mlist.fqdn_listname mailing list.')
+    else:
+        assert(template_name.endswith(':subscribe'))
+        subject = _('Your confirmation is needed to join the '
+                    '$event.mlist.fqdn_listname mailing list.')
     confirm_address = event.mlist.confirm_address(event.token)
     email_address = event.email
     # Send a verification email to the address.
@@ -597,6 +659,13 @@ def handle_SubscriptionConfirmationNeededEvent(event):
     _handle_confirmation_needed_events(event, 'list:user:action:subscribe')
 
 
+@public
+def handle_SubscriptionInvitationNeededEvent(event):
+    if not isinstance(event, SubscriptionInvitationNeededEvent):
+        return
+    _handle_confirmation_needed_events(event, 'list:user:action:invite')
+
+
 @public
 def handle_UnsubscriptionConfirmationNeededEvent(event):
     if not isinstance(event, UnsubscriptionConfirmationNeededEvent):
