@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -37,13 +37,21 @@ class TestSystemConfiguration(unittest.TestCase):
         self.assertIn('http_etag', json)
         del json['http_etag']
         self.assertEqual(json, dict(
+            anonymous_list_keep_headers='^(?!x-) ^x-mailman- '
+                                        '^x-content-filtered-by:\n'
+                                        '^x-topics: ^x-ack: ^x-beenthere: '
+                                        '^x-list-administrivia: ^x-spam-',
             cache_life='7d',
+            check_max_size_on_filtered_message='no',
             default_language='en',
             email_commands_max_lines='10',
+            filter_report='no',
             filtered_messages_are_preservable='no',
+            hold_digest='no',
             html_to_plain_text_command='/usr/bin/lynx -dump $filename',
             layout='testing',
             listname_chars='[-_.0-9a-z]',
+            masthead_threshold='4',
             noreply_address='noreply',
             pending_request_life='3d',
             post_hook='',
