@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2008-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -36,7 +36,8 @@ from mailman.bin.master import Loop as Master
 from mailman.config import config
 from mailman.database.transaction import transaction
 from mailman.email.message import Message
-from mailman.interfaces.member import MemberRole
+from mailman.interfaces.action import Action
+from mailman.interfaces.member import DeliveryStatus, MemberRole
 from mailman.interfaces.messages import IMessageStore
 from mailman.interfaces.styles import IStyleManager
 from mailman.interfaces.usermanager import IUserManager
@@ -417,7 +418,8 @@ class chdir:
 
 
 @public
-def subscribe(mlist, first_name, role=MemberRole.member, email=None):
+def subscribe(mlist, first_name, role=MemberRole.member, email=None,
+              as_user=False):
     """Helper for subscribing a sample person to a mailing list.
 
     Returns the newly created member object.
@@ -432,12 +434,27 @@ def subscribe(mlist, first_name, role=MemberRole.member, email=None):
             address = user_manager.get_address(email)
             if address is None:
                 person = user_manager.create_user(email, full_name)
-                subscription_address = list(person.addresses)[0]
+                set_preferred(person)
+                if as_user:
+                    subscription_object = person
+                else:
+                    subscription_object = person.preferred_address
             else:
-                subscription_address = address
+                if not as_user:
+                    subscription_object = address
+                else:
+                    person = user_manager.create_user(display_name=full_name)
+                    person.link(address)
+                    set_preferred(person)
+                    subscription_object = person
         else:
-            subscription_address = list(person.addresses)[0]
-        mlist.subscribe(subscription_address, role)
+            if as_user:
+                if person.preferred_address is None:
+                    set_preferred(person)
+                subscription_object = person
+            else:
+                subscription_object = list(person.addresses)[0]
+        mlist.subscribe(subscription_object, role)
         roster = mlist.get_roster(role)
         return roster.get_member(email)
 
@@ -584,3 +601,15 @@ def nose2_start_test_run_callback(plugin):
     if (plugin.stderr or
             len(os.environ.get('MM_VERBOSE_TESTLOG', '').strip()) > 0):
         ConfigLayer.stderr = True
+
+
+def set_moderation(mlist, subscriber, action):
+    """Set moderation action for the subscriber of mailing list."""
+    member = mlist.members.get_member(subscriber)
+    member.moderation_action = Action[action]
+
+
+def set_delivery(mlist, subscriber, delivery_status):
+    """Set delivery_status to DeliveryStatus"""
+    member = mlist.members.get_member(subscriber)
+    member.preferences.delivery_status = DeliveryStatus[delivery_status]
